# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/agent.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17\x63ogment/api/agent.proto\x12\ncogmentAPI\x1a\x18\x63ogment/api/common.proto2\xa5\x01\n\x0eServiceActorSP\x12Q\n\x08RunTrial\x12\x1e.cogmentAPI.ActorRunTrialInput\x1a\x1f.cogmentAPI.ActorRunTrialOutput\"\x00(\x01\x30\x01\x12@\n\x07Version\x12\x1a.cogmentAPI.VersionRequest\x1a\x17.cogmentAPI.VersionInfo\"\x00\x62\x06proto3')



_SERVICEACTORSP = DESCRIPTOR.services_by_name['ServiceActorSP']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _SERVICEACTORSP._serialized_start=66
  _SERVICEACTORSP._serialized_end=231
# @@protoc_insertion_point(module_scope)
