# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/directory.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x63ogment/api/directory.proto\x12\ncogmentAPI\x1a\x18\x63ogment/api/common.proto\"m\n\x0fRegisterRequest\x12-\n\x08\x65ndpoint\x18\x01 \x01(\x0b\x32\x1b.cogmentAPI.ServiceEndpoint\x12+\n\x07\x64\x65tails\x18\x02 \x01(\x0b\x32\x1a.cogmentAPI.ServiceDetails\"\xa3\x01\n\rRegisterReply\x12\x30\n\x06status\x18\x01 \x01(\x0e\x32 .cogmentAPI.RegisterReply.Status\x12\x11\n\terror_msg\x18\x02 \x01(\t\x12\x12\n\nservice_id\x18\x03 \x01(\x04\x12\x0e\n\x06secret\x18\x04 \x01(\t\")\n\x06Status\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x06\n\x02OK\x10\x01\x12\n\n\x06\x46\x41ILED\x10\x02\"7\n\x11\x44\x65registerRequest\x12\x12\n\nservice_id\x18\x01 \x01(\x04\x12\x0e\n\x06secret\x18\x02 \x01(\t\"\x83\x01\n\x0f\x44\x65registerReply\x12\x32\n\x06status\x18\x01 \x01(\x0e\x32\".cogmentAPI.DeregisterReply.Status\x12\x11\n\terror_msg\x18\x02 \x01(\t\")\n\x06Status\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x06\n\x02OK\x10\x01\x12\n\n\x06\x46\x41ILED\x10\x02\"`\n\x0eInquireRequest\x12\x14\n\nservice_id\x18\x01 \x01(\x04H\x00\x12-\n\x07\x64\x65tails\x18\x02 \x01(\x0b\x32\x1a.cogmentAPI.ServiceDetailsH\x00\x42\t\n\x07inquiry\"9\n\x0cInquireReply\x12)\n\x04\x64\x61ta\x18\x01 \x01(\x0b\x32\x1b.cogmentAPI.FullServiceData\"\xa3\x01\n\x0fServiceEndpoint\x12\x36\n\x08protocol\x18\x01 \x01(\x0e\x32$.cogmentAPI.ServiceEndpoint.Protocol\x12\x0c\n\x04host\x18\x02 \x01(\t\x12\x0c\n\x04port\x18\x03 \x01(\r\"<\n\x08Protocol\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x08\n\x04GRPC\x10\x01\x12\x0c\n\x08GRPC_SSL\x10\x02\x12\x0b\n\x07\x43OGMENT\x10\x03\"\xaa\x01\n\x0eServiceDetails\x12%\n\x04type\x18\x01 \x01(\x0e\x32\x17.cogmentAPI.ServiceType\x12>\n\nproperties\x18\x02 \x03(\x0b\x32*.cogmentAPI.ServiceDetails.PropertiesEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x81\x01\n\x0f\x46ullServiceData\x12-\n\x08\x65ndpoint\x18\x01 \x01(\x0b\x32\x1b.cogmentAPI.ServiceEndpoint\x12\x12\n\nservice_id\x18\x02 \x01(\x04\x12+\n\x07\x64\x65tails\x18\x03 \x01(\x0b\x32\x1a.cogmentAPI.ServiceDetails*\x82\x02\n\x0bServiceType\x12\x13\n\x0fUNKNOWN_SERVICE\x10\x00\x12\x1c\n\x18TRIAL_LIFE_CYCLE_SERVICE\x10\x01\x12#\n\x1f\x43LIENT_ACTOR_CONNECTION_SERVICE\x10\x02\x12\x11\n\rACTOR_SERVICE\x10\x03\x12\x17\n\x13\x45NVIRONMENT_SERVICE\x10\x04\x12\x14\n\x10PRE_HOOK_SERVICE\x10\x05\x12\x13\n\x0f\x44\x41TALOG_SERVICE\x10\x06\x12\x15\n\x11\x44\x41TASTORE_SERVICE\x10\x07\x12\x1a\n\x16MODEL_REGISTRY_SERVICE\x10\x08\x12\x11\n\rOTHER_SERVICE\x10\x64\x32\xae\x02\n\x0b\x44irectorySP\x12H\n\x08Register\x12\x1b.cogmentAPI.RegisterRequest\x1a\x19.cogmentAPI.RegisterReply\"\x00(\x01\x30\x01\x12N\n\nDeregister\x12\x1d.cogmentAPI.DeregisterRequest\x1a\x1b.cogmentAPI.DeregisterReply\"\x00(\x01\x30\x01\x12\x43\n\x07Inquire\x12\x1a.cogmentAPI.InquireRequest\x1a\x18.cogmentAPI.InquireReply\"\x00\x30\x01\x12@\n\x07Version\x12\x1a.cogmentAPI.VersionRequest\x1a\x17.cogmentAPI.VersionInfo\"\x00\x62\x06proto3')

_SERVICETYPE = DESCRIPTOR.enum_types_by_name['ServiceType']
ServiceType = enum_type_wrapper.EnumTypeWrapper(_SERVICETYPE)
UNKNOWN_SERVICE = 0
TRIAL_LIFE_CYCLE_SERVICE = 1
CLIENT_ACTOR_CONNECTION_SERVICE = 2
ACTOR_SERVICE = 3
ENVIRONMENT_SERVICE = 4
PRE_HOOK_SERVICE = 5
DATALOG_SERVICE = 6
DATASTORE_SERVICE = 7
MODEL_REGISTRY_SERVICE = 8
OTHER_SERVICE = 100


_REGISTERREQUEST = DESCRIPTOR.message_types_by_name['RegisterRequest']
_REGISTERREPLY = DESCRIPTOR.message_types_by_name['RegisterReply']
_DEREGISTERREQUEST = DESCRIPTOR.message_types_by_name['DeregisterRequest']
_DEREGISTERREPLY = DESCRIPTOR.message_types_by_name['DeregisterReply']
_INQUIREREQUEST = DESCRIPTOR.message_types_by_name['InquireRequest']
_INQUIREREPLY = DESCRIPTOR.message_types_by_name['InquireReply']
_SERVICEENDPOINT = DESCRIPTOR.message_types_by_name['ServiceEndpoint']
_SERVICEDETAILS = DESCRIPTOR.message_types_by_name['ServiceDetails']
_SERVICEDETAILS_PROPERTIESENTRY = _SERVICEDETAILS.nested_types_by_name['PropertiesEntry']
_FULLSERVICEDATA = DESCRIPTOR.message_types_by_name['FullServiceData']
_REGISTERREPLY_STATUS = _REGISTERREPLY.enum_types_by_name['Status']
_DEREGISTERREPLY_STATUS = _DEREGISTERREPLY.enum_types_by_name['Status']
_SERVICEENDPOINT_PROTOCOL = _SERVICEENDPOINT.enum_types_by_name['Protocol']
RegisterRequest = _reflection.GeneratedProtocolMessageType('RegisterRequest', (_message.Message,), {
  'DESCRIPTOR' : _REGISTERREQUEST,
  '__module__' : 'cogment.api.directory_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RegisterRequest)
  })
_sym_db.RegisterMessage(RegisterRequest)

RegisterReply = _reflection.GeneratedProtocolMessageType('RegisterReply', (_message.Message,), {
  'DESCRIPTOR' : _REGISTERREPLY,
  '__module__' : 'cogment.api.directory_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RegisterReply)
  })
_sym_db.RegisterMessage(RegisterReply)

DeregisterRequest = _reflection.GeneratedProtocolMessageType('DeregisterRequest', (_message.Message,), {
  'DESCRIPTOR' : _DEREGISTERREQUEST,
  '__module__' : 'cogment.api.directory_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.DeregisterRequest)
  })
_sym_db.RegisterMessage(DeregisterRequest)

DeregisterReply = _reflection.GeneratedProtocolMessageType('DeregisterReply', (_message.Message,), {
  'DESCRIPTOR' : _DEREGISTERREPLY,
  '__module__' : 'cogment.api.directory_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.DeregisterReply)
  })
_sym_db.RegisterMessage(DeregisterReply)

InquireRequest = _reflection.GeneratedProtocolMessageType('InquireRequest', (_message.Message,), {
  'DESCRIPTOR' : _INQUIREREQUEST,
  '__module__' : 'cogment.api.directory_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.InquireRequest)
  })
_sym_db.RegisterMessage(InquireRequest)

InquireReply = _reflection.GeneratedProtocolMessageType('InquireReply', (_message.Message,), {
  'DESCRIPTOR' : _INQUIREREPLY,
  '__module__' : 'cogment.api.directory_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.InquireReply)
  })
_sym_db.RegisterMessage(InquireReply)

ServiceEndpoint = _reflection.GeneratedProtocolMessageType('ServiceEndpoint', (_message.Message,), {
  'DESCRIPTOR' : _SERVICEENDPOINT,
  '__module__' : 'cogment.api.directory_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ServiceEndpoint)
  })
_sym_db.RegisterMessage(ServiceEndpoint)

ServiceDetails = _reflection.GeneratedProtocolMessageType('ServiceDetails', (_message.Message,), {

  'PropertiesEntry' : _reflection.GeneratedProtocolMessageType('PropertiesEntry', (_message.Message,), {
    'DESCRIPTOR' : _SERVICEDETAILS_PROPERTIESENTRY,
    '__module__' : 'cogment.api.directory_pb2'
    # @@protoc_insertion_point(class_scope:cogmentAPI.ServiceDetails.PropertiesEntry)
    })
  ,
  'DESCRIPTOR' : _SERVICEDETAILS,
  '__module__' : 'cogment.api.directory_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ServiceDetails)
  })
_sym_db.RegisterMessage(ServiceDetails)
_sym_db.RegisterMessage(ServiceDetails.PropertiesEntry)

FullServiceData = _reflection.GeneratedProtocolMessageType('FullServiceData', (_message.Message,), {
  'DESCRIPTOR' : _FULLSERVICEDATA,
  '__module__' : 'cogment.api.directory_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.FullServiceData)
  })
_sym_db.RegisterMessage(FullServiceData)

_DIRECTORYSP = DESCRIPTOR.services_by_name['DirectorySP']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _SERVICEDETAILS_PROPERTIESENTRY._options = None
  _SERVICEDETAILS_PROPERTIESENTRY._serialized_options = b'8\001'
  _SERVICETYPE._serialized_start=1166
  _SERVICETYPE._serialized_end=1424
  _REGISTERREQUEST._serialized_start=69
  _REGISTERREQUEST._serialized_end=178
  _REGISTERREPLY._serialized_start=181
  _REGISTERREPLY._serialized_end=344
  _REGISTERREPLY_STATUS._serialized_start=303
  _REGISTERREPLY_STATUS._serialized_end=344
  _DEREGISTERREQUEST._serialized_start=346
  _DEREGISTERREQUEST._serialized_end=401
  _DEREGISTERREPLY._serialized_start=404
  _DEREGISTERREPLY._serialized_end=535
  _DEREGISTERREPLY_STATUS._serialized_start=303
  _DEREGISTERREPLY_STATUS._serialized_end=344
  _INQUIREREQUEST._serialized_start=537
  _INQUIREREQUEST._serialized_end=633
  _INQUIREREPLY._serialized_start=635
  _INQUIREREPLY._serialized_end=692
  _SERVICEENDPOINT._serialized_start=695
  _SERVICEENDPOINT._serialized_end=858
  _SERVICEENDPOINT_PROTOCOL._serialized_start=798
  _SERVICEENDPOINT_PROTOCOL._serialized_end=858
  _SERVICEDETAILS._serialized_start=861
  _SERVICEDETAILS._serialized_end=1031
  _SERVICEDETAILS_PROPERTIESENTRY._serialized_start=982
  _SERVICEDETAILS_PROPERTIESENTRY._serialized_end=1031
  _FULLSERVICEDATA._serialized_start=1034
  _FULLSERVICEDATA._serialized_end=1163
  _DIRECTORYSP._serialized_start=1427
  _DIRECTORYSP._serialized_end=1729
# @@protoc_insertion_point(module_scope)
