# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/hooks.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17\x63ogment/api/hooks.proto\x12\ncogmentAPI\x1a\x18\x63ogment/api/common.proto\"9\n\x0ePreTrialParams\x12\'\n\x06params\x18\x01 \x01(\x0b\x32\x17.cogmentAPI.TrialParams2\x98\x01\n\x0cTrialHooksSP\x12\x46\n\nOnPreTrial\x12\x1a.cogmentAPI.PreTrialParams\x1a\x1a.cogmentAPI.PreTrialParams\"\x00\x12@\n\x07Version\x12\x1a.cogmentAPI.VersionRequest\x1a\x17.cogmentAPI.VersionInfo\"\x00\x62\x06proto3')



_PRETRIALPARAMS = DESCRIPTOR.message_types_by_name['PreTrialParams']
PreTrialParams = _reflection.GeneratedProtocolMessageType('PreTrialParams', (_message.Message,), {
  'DESCRIPTOR' : _PRETRIALPARAMS,
  '__module__' : 'cogment.api.hooks_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.PreTrialParams)
  })
_sym_db.RegisterMessage(PreTrialParams)

_TRIALHOOKSSP = DESCRIPTOR.services_by_name['TrialHooksSP']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _PRETRIALPARAMS._serialized_start=65
  _PRETRIALPARAMS._serialized_end=122
  _TRIALHOOKSSP._serialized_start=125
  _TRIALHOOKSSP._serialized_end=277
# @@protoc_insertion_point(module_scope)
