# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/health.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18\x63ogment/api/health.proto\x12\ncogmentAPI\"%\n\x12HealthCheckRequest\x12\x0f\n\x07service\x18\x01 \x01(\t\"\xde\x01\n\x13HealthCheckResponse\x12=\n\x06status\x18\x01 \x01(\x0e\x32-.cogmentAPI.HealthCheckResponse.ServingStatus\x12\x37\n\x0c\x63ogment_data\x18\x64 \x01(\x0b\x32!.cogmentAPI.CogmentHealthMetadata\"O\n\rServingStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07SERVING\x10\x01\x12\x0f\n\x0bNOT_SERVING\x10\x02\x12\x13\n\x0fSERVICE_UNKNOWN\x10\x03\"\x17\n\x15\x43ogmentHealthMetadata2\x9e\x01\n\x06Health\x12H\n\x05\x43heck\x12\x1e.cogmentAPI.HealthCheckRequest\x1a\x1f.cogmentAPI.HealthCheckResponse\x12J\n\x05Watch\x12\x1e.cogmentAPI.HealthCheckRequest\x1a\x1f.cogmentAPI.HealthCheckResponse0\x01\x62\x06proto3')



_HEALTHCHECKREQUEST = DESCRIPTOR.message_types_by_name['HealthCheckRequest']
_HEALTHCHECKRESPONSE = DESCRIPTOR.message_types_by_name['HealthCheckResponse']
_COGMENTHEALTHMETADATA = DESCRIPTOR.message_types_by_name['CogmentHealthMetadata']
_HEALTHCHECKRESPONSE_SERVINGSTATUS = _HEALTHCHECKRESPONSE.enum_types_by_name['ServingStatus']
HealthCheckRequest = _reflection.GeneratedProtocolMessageType('HealthCheckRequest', (_message.Message,), {
  'DESCRIPTOR' : _HEALTHCHECKREQUEST,
  '__module__' : 'cogment.api.health_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.HealthCheckRequest)
  })
_sym_db.RegisterMessage(HealthCheckRequest)

HealthCheckResponse = _reflection.GeneratedProtocolMessageType('HealthCheckResponse', (_message.Message,), {
  'DESCRIPTOR' : _HEALTHCHECKRESPONSE,
  '__module__' : 'cogment.api.health_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.HealthCheckResponse)
  })
_sym_db.RegisterMessage(HealthCheckResponse)

CogmentHealthMetadata = _reflection.GeneratedProtocolMessageType('CogmentHealthMetadata', (_message.Message,), {
  'DESCRIPTOR' : _COGMENTHEALTHMETADATA,
  '__module__' : 'cogment.api.health_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.CogmentHealthMetadata)
  })
_sym_db.RegisterMessage(CogmentHealthMetadata)

_HEALTH = DESCRIPTOR.services_by_name['Health']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _HEALTHCHECKREQUEST._serialized_start=40
  _HEALTHCHECKREQUEST._serialized_end=77
  _HEALTHCHECKRESPONSE._serialized_start=80
  _HEALTHCHECKRESPONSE._serialized_end=302
  _HEALTHCHECKRESPONSE_SERVINGSTATUS._serialized_start=223
  _HEALTHCHECKRESPONSE_SERVINGSTATUS._serialized_end=302
  _COGMENTHEALTHMETADATA._serialized_start=304
  _COGMENTHEALTHMETADATA._serialized_end=327
  _HEALTH._serialized_start=330
  _HEALTH._serialized_end=488
# @@protoc_insertion_point(module_scope)
