// Copyright 2022 AI Redefined Inc. <dev+cogment@ai-r.com>
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package cogmentAPI;

import "cogment/api/common.proto";

service TrialHooksSP {
  // Expected headers:
  // - trial-id
  // - user-id
  rpc OnPreTrial(PreTrialParams) returns (PreTrialParams) {}
  
  // Expected headers: None
  rpc Version(VersionRequest) returns (VersionInfo) {}
}

message PreTrialParams {
  TrialParams params = 1;
}
